"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.findLeastCommonMultiple = void 0;
exports.mergeSchemas = mergeSchemas;
exports.validatorPlugin = void 0;
var _CommonTypings = require("@ui-schema/ui-schema/CommonTypings");
var _createMap = require("@ui-schema/ui-schema/createMap");
var _immutable = require("immutable");
function _toConsumableArray(r) { return _arrayWithoutHoles(r) || _iterableToArray(r) || _unsupportedIterableToArray(r) || _nonIterableSpread(); }
function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _iterableToArray(r) { if ("undefined" != typeof Symbol && null != r[Symbol.iterator] || null != r["@@iterator"]) return Array.from(r); }
function _arrayWithoutHoles(r) { if (Array.isArray(r)) return _arrayLikeToArray(r); }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
var findMax = function findMax(currentVal, newVal) {
  if (typeof newVal === 'number') {
    return typeof currentVal === 'number' ? Math.max(currentVal, newVal) : newVal;
  }
  return currentVal;
};
var findMin = function findMin(currentVal, newVal) {
  if (typeof newVal === 'number') {
    return typeof currentVal === 'number' ? Math.min(currentVal, newVal) : newVal;
  }
  return currentVal;
};
var isFiniteNumber = function isFiniteNumber(n) {
  return typeof n === 'number' && Number.isFinite(n);
};
var getScale = function getScale(n) {
  var s = n.toString().split('.')[1];
  return s ? Math.pow(10, s.length) : 1;
};
var _getGCD = function getGCD(a, b) {
  return b === 0n ? a : _getGCD(b, a % b);
};
var getLCM = function getLCM(a, b) {
  return a * b / _getGCD(a, b);
};
var findLeastCommonMultiple = exports.findLeastCommonMultiple = function findLeastCommonMultiple() {
  for (var _len = arguments.length, values = new Array(_len), _key = 0; _key < _len; _key++) {
    values[_key] = arguments[_key];
  }
  if (!values.every(isFiniteNumber)) throw new Error('Non-finite input');
  if (values.includes(0)) return 0;
  var scaleFactors = values.map(getScale);
  var overallScale = scaleFactors.reduce(function (a, b) {
    return a * b;
  }, 1);
  var intValues = values.map(function (v) {
    return BigInt(Math.round(v * overallScale));
  });
  return Number(intValues.reduce(function (a, b) {
    return getLCM(a, b);
  })) / overallScale;
};
function mergeSchemas(baseSchema) {
  var mergedSchema = baseSchema instanceof _immutable.Map ? baseSchema : (0, _immutable.OrderedMap)(baseSchema);
  if (mergedSchema.has('allOf')) {
    mergedSchema = mergedSchema.remove('allOf');
  }
  var typeConflict = false;
  for (var _len2 = arguments.length, appliedSchemas = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    appliedSchemas[_key2 - 1] = arguments[_key2];
  }
  var _loop = function _loop() {
    var appliedSchema = _appliedSchemas[_i];
    if (!appliedSchema || !(0, _CommonTypings.isSomeSchema)(appliedSchema)) return 1;
    var tmpSchema = mergedSchema;
    appliedSchema.forEach(function (value, key) {
      switch (key) {
        case 'required':
          {
            var baseRequired = tmpSchema.get('required', (0, _immutable.List)());
            var appliedRequired = value;
            tmpSchema = tmpSchema.set('required', baseRequired.toSet().union(appliedRequired.toSet()).toList());
            break;
          }
        case 'enum':
          {
            var baseEnum = tmpSchema.get('enum');
            var appliedEnum = value;
            if (baseEnum) {
              var baseSet = baseEnum.toSet();
              var appliedSet = appliedEnum.toSet();
              tmpSchema = tmpSchema.set('enum', baseSet.intersect(appliedSet).toList());
            } else {
              tmpSchema = tmpSchema.set('enum', appliedEnum);
            }
            break;
          }
        case 'type':
          {
            if (typeConflict) {
              break;
            }
            var baseType = tmpSchema.get('type');
            var appliedType = value;
            var baseTypes = (0, _immutable.Set)(_immutable.List.isList(baseType) ? baseType.toArray() : [baseType]);
            var appliedTypes = (0, _immutable.Set)(_immutable.List.isList(appliedType) ? appliedType.toArray() : [appliedType]);
            if (tmpSchema.has('type')) {
              baseTypes = baseTypes.intersect(appliedTypes);
            } else {
              baseTypes = appliedTypes;
            }
            if (baseTypes.size === 1) {
              tmpSchema = tmpSchema.set('type', baseTypes.first());
            } else if (baseTypes.size === 0) {
              typeConflict = true;
            } else {
              tmpSchema = tmpSchema.set('type', baseTypes.toList());
            }
            break;
          }
        case 'minLength':
        case 'minItems':
        case 'minimum':
        case 'exclusiveMinimum':
          tmpSchema = tmpSchema.set(key, findMax(tmpSchema.get(key), value));
          break;
        case 'maxLength':
        case 'maxItems':
        case 'maximum':
        case 'exclusiveMaximum':
          tmpSchema = tmpSchema.set(key, findMin(tmpSchema.get(key), value));
          break;
        case 'multipleOf':
          {
            var baseMultipleOf = tmpSchema.get('multipleOf');
            if (typeof baseMultipleOf === 'number' && typeof value === 'number') {
              tmpSchema = tmpSchema.set('multipleOf', findLeastCommonMultiple(baseMultipleOf, value));
            } else {
              tmpSchema = tmpSchema.set('multipleOf', value);
            }
            break;
          }
        case 'uniqueItems':
          tmpSchema = tmpSchema.set('uniqueItems', tmpSchema.get('uniqueItems', false) || value);
          break;
        case 'properties':
          {
            var baseProperties = tmpSchema.get('properties', (0, _immutable.OrderedMap)());
            var appliedProperties = value;
            var newProperties = baseProperties;
            appliedProperties.forEach(function (subSchema, propKey) {
              if (newProperties.has(propKey)) {
                newProperties = newProperties.updateIn([propKey, 'allOf'], function () {
                  var allOf = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _immutable.List)();
                  return allOf.push(subSchema);
                });
              } else {
                newProperties = newProperties.set(propKey, subSchema);
              }
            });
            tmpSchema = tmpSchema.set('properties', newProperties);
            break;
          }
        case 'patternProperties':
          {
            var basePatterns = tmpSchema.get('patternProperties', (0, _immutable.OrderedMap)());
            var appliedPatterns = value;
            var newPatterns = basePatterns;
            appliedPatterns.forEach(function (subSchema, pattern) {
              if (newPatterns.has(pattern)) {
                newPatterns = newPatterns.updateIn([pattern, 'allOf'], function () {
                  var allOf = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _immutable.List)();
                  return allOf.push(subSchema);
                });
              } else {
                newPatterns = newPatterns.set(pattern, subSchema);
              }
            });
            tmpSchema = tmpSchema.set('patternProperties', newPatterns);
            break;
          }
        case 'items':
        case 'prefixItems':
          {
            var isTuple = _immutable.List.isList(value) && value.every(_CommonTypings.isSomeSchema);
            var base = tmpSchema.get(key);
            if (isTuple && _immutable.List.isList(base)) {
              var merged = base.map(function (baseItem, idx) {
                var appliedItem = value.get(idx);
                return appliedItem ? (0, _immutable.OrderedMap)({
                  allOf: (0, _immutable.List)([baseItem, appliedItem])
                }) : baseItem;
              });
              tmpSchema = tmpSchema.set(key, merged);
            } else {
              if (_immutable.Map.isMap(value) && _immutable.Map.isMap(base)) {
                tmpSchema = tmpSchema.set(key, (0, _immutable.OrderedMap)({
                  allOf: (0, _immutable.List)([base, value])
                }));
              } else {
                tmpSchema = tmpSchema.set(key, value);
              }
            }
            break;
          }
        case 'additionalProperties':
          if (value === false) {
            tmpSchema = tmpSchema.set('additionalProperties', false);
          } else if (_immutable.Map.isMap(value) && tmpSchema.get('additionalProperties') !== false) {
            var baseAddProps = tmpSchema.get('additionalProperties');
            if (_immutable.Map.isMap(baseAddProps)) {
              tmpSchema = tmpSchema.set('additionalProperties', (0, _immutable.OrderedMap)({
                allOf: (0, _immutable.List)([baseAddProps, value])
              }));
            } else {
              tmpSchema = tmpSchema.set('additionalProperties', value);
            }
          } else if (!tmpSchema.has('additionalProperties')) {
            tmpSchema = tmpSchema.set('additionalProperties', value);
          }
          break;
        case 'const':
          if (tmpSchema.has('const') && !(0, _immutable.is)(tmpSchema.get('const'), value)) {
            tmpSchema = tmpSchema.update('allOf', function () {
              var allOf = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _immutable.List)();
              return allOf.push((0, _immutable.OrderedMap)({
                "const": value
              }));
            });
          } else {
            tmpSchema = tmpSchema.set('const', value);
          }
          break;
        case 'allOf':
        case 'not':
        case 'if':
        case 'then':
        case 'else':
        case 'dependentSchemas':
        case '$defs':
        case 'definitions':
          break;
        case 'anyOf':
          {
            var baseAnyOf = tmpSchema.get('anyOf', (0, _immutable.List)());
            var appliedAnyOf = key === 'anyOf' ? value : (0, _immutable.List)();
            if (appliedAnyOf.size > 0) {
              if (baseAnyOf.size > 0) {
                tmpSchema = tmpSchema.set('anyOf', (0, _immutable.List)([(0, _immutable.OrderedMap)({
                  anyOf: baseAnyOf
                }), (0, _immutable.OrderedMap)({
                  anyOf: appliedAnyOf
                })]));
              } else {
                tmpSchema = tmpSchema.set('anyOf', appliedAnyOf);
              }
            }
            break;
          }
        case 'oneOf':
          {
            var baseOneOf = tmpSchema.get('oneOf', (0, _immutable.List)());
            var appliedOneOf = key === 'oneOf' ? value : (0, _immutable.List)();
            if (appliedOneOf.size > 0) {
              if (baseOneOf.size > 0) {
                tmpSchema = tmpSchema.set('oneOf', (0, _immutable.List)([(0, _immutable.OrderedMap)({
                  oneOf: baseOneOf
                }), (0, _immutable.OrderedMap)({
                  oneOf: appliedOneOf
                })]));
              } else {
                tmpSchema = tmpSchema.set('oneOf', appliedOneOf);
              }
            }
            break;
          }
        case '$ref':
          break;
        default:
          if (tmpSchema.has(key) && _immutable.Map.isMap(tmpSchema.get(key)) && _immutable.Map.isMap(value)) {
            tmpSchema = tmpSchema.set(key, tmpSchema.get(key).mergeDeep(value));
          } else {
            tmpSchema = tmpSchema.set(key, value);
          }
          break;
      }
    });
    mergedSchema = tmpSchema;
  };
  for (var _i = 0, _appliedSchemas = appliedSchemas; _i < _appliedSchemas.length; _i++) {
    if (_loop()) continue;
  }
  return mergedSchema;
}
var validatorPlugin = exports.validatorPlugin = {
  handle: function handle(props) {
    var _props$storeKeys, _props$storeKeys2;
    if (!props.validate) return {};
    var ownKey = (_props$storeKeys = props.storeKeys) === null || _props$storeKeys === void 0 ? void 0 : _props$storeKeys.last();
    var result = props.validate(props.schema, props.value, {
      instanceLocation: ((_props$storeKeys2 = props.storeKeys) === null || _props$storeKeys2 === void 0 ? void 0 : _props$storeKeys2.toArray()) || [],
      keywordLocation: [],
      instanceKey: ownKey,
      parentSchema: props.parentSchema,
      resource: props.resource
    });
    var applicableSchema = mergeSchemas.apply(void 0, [props.schema].concat(_toConsumableArray(result.applied || [])));
    if (result.valid) {
      return {
        valid: true,
        schema: applicableSchema
      };
    }
    var errors = (0, _createMap.createOrdered)(result.errors);
    return {
      valid: false,
      errors: errors,
      schema: applicableSchema
    };
  }
};