"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validateMinMaxArray = void 0;
var _MinMaxValidatorErrorCodes = require("./MinMaxValidatorErrorCodes.cjs");
var _immutable = require("immutable");
var validateMinMaxArray = exports.validateMinMaxArray = function validateMinMaxArray(schema, value, output) {
  if (typeof value === 'undefined') return;
  var minItems = schema.get('minItems');
  var maxItems = schema.get('maxItems');
  if (minItems) {
    if (_immutable.List.isList(value)) {
      if (value.size < minItems) {
        output.addError({
          error: _MinMaxValidatorErrorCodes.ERROR_MIN_LENGTH,
          context: {
            min: minItems
          }
        });
      }
    } else if (Array.isArray(value)) {
      if (value.length < minItems) {
        output.addError({
          error: _MinMaxValidatorErrorCodes.ERROR_MIN_LENGTH,
          context: {
            min: minItems
          }
        });
      }
    }
  }
  if (maxItems) {
    if (_immutable.List.isList(value)) {
      if (value.size > maxItems) {
        output.addError({
          error: _MinMaxValidatorErrorCodes.ERROR_MAX_LENGTH,
          context: {
            max: maxItems
          }
        });
      }
    } else if (Array.isArray(value)) {
      if (value.length > maxItems) {
        output.addError({
          error: _MinMaxValidatorErrorCodes.ERROR_MAX_LENGTH,
          context: {
            max: maxItems
          }
        });
      }
    }
  }
};