"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validateMinMaxNumber = void 0;
var _MinMaxValidatorErrorCodes = require("./MinMaxValidatorErrorCodes.cjs");
var validateMinMaxNumber = exports.validateMinMaxNumber = function validateMinMaxNumber(schema, value, output) {
  if (typeof value === 'undefined') return;
  var minimum = schema.get('minimum');
  var exclusiveMinimum = schema.get('exclusiveMinimum');
  var maximum = schema.get('maximum');
  var exclusiveMaximum = schema.get('exclusiveMaximum');
  if (typeof value === 'number') {
    if (typeof minimum === 'number') {
      if (typeof exclusiveMinimum === 'boolean') {
        if (exclusiveMinimum && value <= minimum) {
          output.addError({
            error: _MinMaxValidatorErrorCodes.ERROR_MIN_LENGTH,
            context: {
              exclMin: minimum
            }
          });
        }
      } else if (value < minimum) {
        output.addError({
          error: _MinMaxValidatorErrorCodes.ERROR_MIN_LENGTH,
          context: {
            min: minimum
          }
        });
      }
    }
    if (typeof exclusiveMinimum === 'number') {
      if (value <= exclusiveMinimum) {
        output.addError({
          error: _MinMaxValidatorErrorCodes.ERROR_MIN_LENGTH,
          context: {
            exclMin: exclusiveMinimum
          }
        });
      }
    }
    if (typeof maximum === 'number') {
      if (typeof exclusiveMaximum === 'boolean') {
        if (value >= maximum) {
          output.addError({
            error: _MinMaxValidatorErrorCodes.ERROR_MAX_LENGTH,
            context: {
              exclMax: maximum
            }
          });
        }
      } else if (value > maximum) {
        output.addError({
          error: _MinMaxValidatorErrorCodes.ERROR_MAX_LENGTH,
          context: {
            max: maximum
          }
        });
      }
    }
    if (typeof exclusiveMaximum === 'number') {
      if (value >= exclusiveMaximum) {
        output.addError({
          error: _MinMaxValidatorErrorCodes.ERROR_MAX_LENGTH,
          context: {
            exclMax: exclusiveMaximum
          }
        });
      }
    }
  }
};