import { toPointer } from '@ui-schema/json-pointer/toPointer';
import { List, Map as ImmutableMap, Record } from 'immutable';
export const ERROR_ADDITIONAL_PROPERTIES = 'additional-properties';
export const validateObject = (schema, value, params) => {
  const isRealObject = typeof value !== 'undefined' && (typeof value === 'object' && value !== null || ImmutableMap.isMap(value) || Record.isRecord(value)) && !(List.isList(value) || Array.isArray(value));
  const properties = schema.get('properties');
  if (schema.get('additionalProperties') === false && properties && isRealObject) {
    let hasAdditional = false;
    const valueProperties = ImmutableMap.isMap(value) || Record.isRecord(value) ? value.toSeq().keySeq() : Object.keys(value);
    const schemaProperties = properties.keySeq();
    valueProperties.forEach(key => {
      if (schemaProperties?.indexOf(key) === -1) hasAdditional = true;
    });
    if (hasAdditional) {
      params.output.addError({
        error: ERROR_ADDITIONAL_PROPERTIES,
        keywordLocation: toPointer([...params.keywordLocation, 'additionalProperties']),
        instanceLocation: toPointer(params.instanceLocation)
      });
    }
  }
  const propertyNames = schema.get('propertyNames');
  if (propertyNames && isRealObject) {
    const keys = ImmutableMap.isMap(value) || Record.isRecord(value) ? value.toSeq().keySeq() : Object.keys(value);
    keys.forEach(key => {
      params.validate(propertyNames.set('type', 'string'), key, {
        ...params,
        keywordLocation: [...params.keywordLocation, 'propertyNames']
      });
    });
  }
  if (params.recursive && properties) {
    properties.forEach((subSchema, key) => {
      if (!subSchema || typeof key !== 'string') return;
      const val = isRealObject ? ImmutableMap.isMap(value) || Record.isRecord(value) ? value.get(key) : value[key] : undefined;
      params.validate(subSchema, val, {
        ...params,
        parentSchema: schema,
        keywordLocation: [...params.keywordLocation, 'properties', key],
        instanceLocation: [...params.instanceLocation, key],
        instanceKey: key,
        context: {}
      });
      params.context.evaluatedProperties ||= new Map();
      if (!params.context.evaluatedProperties.has(key)) {
        params.context.evaluatedProperties.set(key, []);
      }
      params.context.evaluatedProperties.get(key).push(subSchema);
    });
  } else {
    properties?.forEach((subSchema, key) => {
      if (typeof key !== 'string') return;
      params.context.evaluatedProperties ||= new Map();
      if (!params.context.evaluatedProperties.has(key)) {
        params.context.evaluatedProperties.set(key, []);
      }
      params.context.evaluatedProperties.get(key).push(subSchema);
    });
  }
};
export const objectValidator = {
  types: ['object'],
  validate: (schema, value, params) => {
    if (!schema) return;
    validateObject(schema, value, params);
  }
};