import { List, Map } from 'immutable';
import { schemaTypeIs, schemaTypeIsNumeric } from '@ui-schema/ui-schema/schemaTypeIs';
export const ERROR_NOT_SET = 'required-not-set';
export const checkValueExists = (type, value) => {
  const valType = typeof value;
  if (valType === 'undefined') {
    return false;
  }
  if (schemaTypeIs(type, 'string') && valType === 'string') {
    return value !== '';
  } else if (valType === 'string' && schemaTypeIsNumeric(type)) {
    return value !== '';
  }
  return true;
};
export const requiredValidator = {
  handle: ({
    parentSchema,
    schema,
    storeKeys,
    value,
    errors,
    valid
  }) => {
    const requiredList = parentSchema?.get('required');
    if (!schema || !requiredList || !List.isList(requiredList) || !storeKeys) {
      return {
        required: false
      };
    }
    const ownKey = storeKeys.last();
    if (typeof ownKey !== 'string' || !requiredList.contains(ownKey)) {
      return {
        required: false
      };
    }
    const type = schema.get('type');
    if (!checkValueExists(type, value)) {
      valid = false;
      errors = (errors || List([])).push(Map({
        error: ERROR_NOT_SET
      }));
    }
    return {
      errors,
      valid,
      required: true
    };
  }
};